-module(boss).
-behaviour(supervisor).
-behaviour(application).

-export([start/0, start/2, stop/0, stop/1, start_link/1, stop_child/1,
        children/0]).

-export([init/1]).

start() ->
    application:start(?MODULE).

start(_Type, StartArgs) ->
    start_link(StartArgs).

stop() ->
    application:stop(?MODULE).

stop(_State) ->
    ok.

start_link(StartArgs) ->
    supervisor:start_link({local, ?MODULE}, ?MODULE, StartArgs).

stop_child(Id) ->
    supervisor:terminate_child(?MODULE, Id).

children() ->
    supervisor:which_children(?MODULE).

init([Count, MilliSecs]) ->
    Nanny = {nanny,
                    {nanny, start_link, []},
                    permanent, infinity, supervisor, [nanny]},

    Manage = {manage,
                    {manage, start_link, [Count]},
                    permanent, 5000, worker, [manage]},

    Delay = {delay,
                {delay, start_link, [MilliSecs]},
                permanent, 15000, worker, [delay]},

    {ok,{{one_for_one, 3, 10}, 
            [Nanny, Manage, Delay]}}.
