-module(delay).
-behaviour(gen_server).

-export([start_link/1]).

-export([init/1,
        handle_call/3,
        handle_cast/2,
        handle_info/2,
        terminate/2,
        code_change/3]).

-record(state, {millisecs}).

start_link(MilliSecs) ->
    gen_server:start_link(?MODULE, [MilliSecs], []).


init([MilliSecs]) ->
    process_flag(trap_exit, true),
    {ok, #state{millisecs=MilliSecs}}.

handle_call(Req, _From, St) ->
    {reply, {error, Req}, St}.

handle_cast(_Req, St) ->
    {noreply, St}.

handle_info(_Req, St) ->
    {noreply, St}.

terminate(Reason, #state{millisecs=MilliSecs}) ->
    error_logger:info_report([{stopping, ?MODULE}, {reason, Reason}]),
    timer:sleep(MilliSecs),
    error_logger:info_report([{delaying_completed, MilliSecs}]),
    ok.

code_change(_OldVsn, St, _Extra) ->
    {noreply, St}.
