-module(manage).
-behaviour(gen_server).

-export([start_link/1]).

-export([init/1,
        handle_call/3,
        handle_cast/2,
        handle_info/2,
        terminate/2,
        code_change/3]).


start_link(Count) ->
    gen_server:start_link({local, ?MODULE}, ?MODULE, [Count], []).


init([Count]) ->
    process_flag(trap_exit, true),

    case start_children(Count) of
        {ok, 0} ->
            {stop, no_children};
        {ok, _} ->
            error_logger:info_report([{started, manage}]),
            {ok, nil}
    end.

handle_call(Req, _From, St) ->
    {reply, {error, Req}, St}.

handle_cast(_Req, St) ->
    {noreply, St}.

handle_info(_Req, St) ->
    {noreply, St}.

terminate(Reason, _St) ->
    error_logger:info_report([{stopping, manage}, {reason, Reason}]),
    ok.

code_change(_OldVsn, St, _Extra) ->
    {noreply, St}.

start_children(Count) ->
    start_children(Count, 0).

start_children(0, N) ->
    {ok, N};
start_children(Count, N) ->
    case nanny:start_child() of
        {error, _} ->
            start_children(Count -1, N);
        {ok, _} ->
            start_children(Count -1, N+1)
    end.
